// AABB - axis aligned bounding box class
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// under GPL or new BSD license
// v 0 (01.11.02): initial release

#ifndef __AABB_H
#define __AABB_H

#include "Vec3.h"

class AABB
            // Axis-aligned bounding box (3D)
{
public:
    Vec3 min,max;


public:
    AABB(){}         // For performance, nothing is cleared
    ~AABB(){}

    void Clear()
    {
        min.Reset();
        max.Reset();
    }

    void SetMinMax(Vec3 aabb_min,Vec3 aabb_max)
    {
        min=aabb_min;
        max=aabb_max;
    }

    const Vec3 GetCenter() const
    {
        return Vec3::MakeAverage(min,max);
    }

    const float GetRadius() const
    {
        return (max-min).length()*0.5f;
    }

    float GetVolume() const
    {
        return (max.x-min.x)*(max.y-min.y)*(max.z-min.z);
    }

    // Creating an AABB from other primitives
    void   MakeFromRay(const Vec3 *org,const Vec3 *dir,float len);


    float GetWidth(){ return max.x-min.x; }
    float GetHeight(){ return max.y-min.y; }
    float GetDepth(){ return max.z-min.z; }

    bool   Overlaps(AABB *aabb)
    // Returns TRUE if the boxes overlap
    {
        return(aabb->min.x<max.x&&aabb->max.x>min.x&&
               aabb->min.y<max.y&&aabb->max.y>min.y&&
               aabb->min.z<max.z&&aabb->max.z>min.z);
    }
    bool   Contains(AABB *aabb)
    // Returns TRUE if 'aabb' is completely contained in this object
    {
        return(aabb->min.x>=min.x&&aabb->max.x<=max.x&&
               aabb->min.y>=min.y&&aabb->max.y<=max.y&&
               aabb->min.z>=min.z&&aabb->max.z<=max.z);
    }


};

#endif

