#ifndef MODEL3D_H
#define MODEL3D_H
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// based on code by Digiben
// under GPL or new BSD license
// v 0 (21.10.02): initial release
// v 1 (01.11.02): AABB stuff added
#include <vector>
#include <gl\gl.h>

using namespace _STL;


//#include "CVector3.h"
#include "Vec3.h"
#include "aabb.hpp"
#include "sphere.hpp"

class	IrisSection
{
public:
    IrisSection(){_indice = NULL;};
    ~IrisSection(){delete _indice ;};
    int				GetNumIndice() { return _num_indice;}
    void    		SetNumIndice(int i)
    {
        _num_indice = i;
        if (_indice != NULL) delete [] _indice;
        _indice = new unsigned short[i];
    }
unsigned short		GetIndice(int i){return (_indice[i]);}
    unsigned short		*GetIndicePtr(void){return (_indice);}
    // protected:
    unsigned short	*_indice;
    int				_num_indice;
};
// This is our 2D point class.  This will be used to store the UV coordinates.
class CVector2
{
public:
    float x, y;
};

// This is our face structure.  This is is used for indexing into the vertex
// and texture coordinate arrays.  From this information we know which vertices
// from our vertex array go to which face, along with the correct texture coordinates.
struct tFace
{
    int vertIndex[3];			// indicies for the verts that make up this triangle
    int coordIndex[3];			// indicies for the tex coords to texture this face
};

struct tStrip
{
    unsigned short	*_indice;
    int				_num_indice;
};

// This holds the information for a material.  It may be a texture map of a color.
// Some of these are not used, but I left them because you will want to eventually
// read in the UV tile ratio and the UV tile offset for some models.
struct tMaterialInfo
{
    char  strName[255];			// The texture name
    char  strFile[255];			// The texture file name (If this is set it's a texture map)
    char  color[3];				// The color of the object (R, G, B)
    int   textureId;				// the texture ID
    int   materialId; //nr of the material in the model
    // 	float uTile;				// u tiling of texture  (Currently not used)
    // 	float vTile;				// v tiling of texture	(Currently not used)
    // 	float uOffset;			    // u offset of texture	(Currently not used)
    // 	float vOffset;				// v offset of texture	(Currently not used)
} ;

// This holds all the information for our model/scene.
// You should eventually turn into a robust class that
// has loading/drawing/querying functions like:
// LoadModel(...); DrawObject(...); DrawModel(...); DestroyModel(...);
struct t3DObject
{
    //Bounding Sphere
    SPHERE	sphere;
    //AABB Bounding Box
   	AABB	box;
     ///////////////////////////////
    int  numOfVerts;			// The number of verts in the model
    int  numOfFaces;			// The number of faces in the model
    int  numOfSections;			// The number of Sections/Strips in the model
    int  numTexVertex;			// The number of texture coordinates
    int  materialID;			// The texture ID to use, which is the index into our texture array
    bool bHasTexture;			// This is TRUE if there is a texture map for this object
    char strName[255];			// The name of the object
    Vec3  *pVerts;			// The object's vertices
    CVector2  *pTexVerts;		// The texture's UV coordinates
    uint32 *color;			// The objects vertices colors
#ifdef USE_NORMALS
    Vec3  *pNormals;		// The object's normals
#endif

    tFace *pFaces;				// The Triangle faces information of the object
    IrisSection *pStrip;		// The strips information of the object
    // void *pTriInfo;
#ifdef USE_ARRAYS
    /////// * /////////// * /////////// * NEW * /////// * /////////// * /////////// *
    uint32     *pIndices;		// The straight face indices in a row, used for vertex arrays
    /////// * /////////// * /////////// * NEW * /////// * /////////// * /////////// *
#endif
};

// This holds our model information.  This should also turn into a robust class.
// We use STL's (Standard Template Library) vector class to ease our link list burdens. :)
struct t3DModel
{
    int numOfObjects;					// The number of objects in the model
    int numOfMaterials;					// The number of materials for the model
    int numOfIndicies;
    vector<tMaterialInfo> pMaterials;	// The list of material information (Textures and colors)
    vector<t3DObject> pObject;			// The object list for our model
};


#endif
